using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Collections;
using System.Windows.Data;
using System.Windows.Markup;
using System.ComponentModel.Composition;

using Microsoft.LightSwitch.Details;
using Microsoft.LightSwitch.Model;
using Microsoft.LightSwitch.Presentation;
using Microsoft.LightSwitch.Details.Client;

namespace SimpleDataGridExtension.Presentation.Controls
{
    public partial class SimpleDataGrid : UserControl
    {
        public SimpleDataGrid()
        {
            InitializeComponent();
            this.SetBinding(ContentItemProperty, new Binding());
        }

        /// <summary>
        /// ContentItem property is bound to the IContentItem inside the 
        /// LightSwitch screen layout tree. 
        /// This is created to monitor if the DataContext of the control is changed.
        /// </summary>
        public IContentItem ContentItem
        {
            get { return (IContentItem)GetValue(ContentItemProperty); }
            set { SetValue(ContentItemProperty, value); }
        }

        public static readonly DependencyProperty ContentItemProperty =
            DependencyProperty.Register("ContentItem",
            typeof(IContentItem), typeof(SimpleDataGrid),
            new PropertyMetadata(OnContentItemChanged));

        private static void OnContentItemChanged(DependencyObject d,
            DependencyPropertyChangedEventArgs e)
        {
            // When the ContentItem is changed, we reset the internal data binding
            ((SimpleDataGrid)d).SetContentDataBinding();
        }

        /// <summary>
        /// SetContentDataBinding is called when the control is bound to the screen layout tree, 
        /// or ContentItem is ever changed.
        /// We calculate which entity property should be used, and create DataBinding for 
        /// EntityPropertyProperty and IsComputedProperty in this method.
        /// </summary>
        private void SetContentDataBinding()
        {
            if (ContentItem != null)
            {
                // Get the collection that was passed to the control
                IScreenCollectionProperty boundCollection =
                    (IScreenCollectionProperty)ContentItem.Screen.Details.Properties[ContentItem.BindingPath];

                // Get the Model Members of the collection
                var ModelMembers =
                    ContentItem.ChildItems[0].ResultingDataType.Members.OfType<IDataTypeMemberDefinition>();

                // Loop thru each Model Members
                foreach (var item in ModelMembers)
                {
                    // Only use the properties that are EntityProperties
                    if (item.GetType().FullName == "Microsoft.LightSwitch.Model.Storage.EntityProperty")
                    {
                        // Create a DataGrid column
                        DataGridTextColumn objDataGridTextColumn = new DataGridTextColumn();
                        objDataGridTextColumn.Header = item.Name.ToString();
                        objDataGridTextColumn.Binding = new Binding(item.Name.ToString());
                        dgItems.Columns.Add(objDataGridTextColumn);
                    }
                }

                // Turns off Auto Generate Columns on the DastaGrid
                dgItems.AutoGenerateColumns = false;
                // Bind the DataGrid
                dgItems.ItemsSource = (IEnumerable)boundCollection.Value;
            }
        }
    }

    [Export(typeof(IControlFactory))]
    [ControlFactory("SimpleDataGridExtension:SimpleDataGrid")]
    internal class SimpleDataGridFactory : IControlFactory
    {
        #region IControlFactory Members

        public DataTemplate DataTemplate
        {
            get
            {
                if (null == this.dataTemplate)
                {
                    this.dataTemplate = XamlReader.Load(SimpleDataGridFactory.ControlTemplate) as DataTemplate;
                }
                return this.dataTemplate;
            }
        }

        public DataTemplate GetDisplayModeDataTemplate(IContentItem contentItem)
        {
            return null;
        }

        #endregion

        #region Private Fields

        private DataTemplate dataTemplate;

        #endregion

        #region Constants

        private const string ControlTemplate =
            "<DataTemplate" +
            " xmlns=\"http://schemas.microsoft.com/winfx/2006/xaml/presentation\"" +
            " xmlns:x=\"http://schemas.microsoft.com/winfx/2006/xaml\"" +
            " xmlns:ctl=\"clr-namespace:SimpleDataGridExtension.Presentation.Controls;assembly=SimpleDataGridExtension.Client\">" +
            "<ctl:SimpleDataGrid/>" +
            "</DataTemplate>";

        #endregion
    }
}
